<?php
// $Horde: turba/lib/Driver.php,v 1.8 2000/08/14 18:58:10 chuck Exp $

require_once './lib/Turba.php';

/**
 * The Turba_Driver:: class provides a common abstracted interface to the
 * various directory search drivers.  It includes functions for searching,
 * adding, removing, and modifying directory entries.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.8 $
 * @since   Turba 0.0.1
 */
class Turba_Driver {

    // {{{ properties

    /** String holding the name of the current driver. */
    var $type = '';

    /** Hash holding the driver's additional parameters. */
    var $params = array();

    /** String holding the current error. */
    var $errstr = '';

    /** Integer holding the current error code. */
    var $errno = '';

    // }}}

    // {{{
    /**
     * Attempts to return a concrete Turba_Driver instance based on $driver.
     *
     * @param $driver   The type of Turba_Driver subclass to return.  The
     *                  code is dynamically included.
     * @param $params   Hash containing additional paramters to be passed to
     *                  the subclass's constructor.
     *
     * @return          The newly created concrete Turba_Driver instance, or
     *                  false on an error.
     */
    function factory($driver, $params)
    {
        $classfile = dirname(__FILE__) . '/Driver/' . $driver . '.php';

        if (@is_readable($classfile)) {
            include_once $classfile;
            $class = 'Turba_Driver_' . $driver;
            return new $class($params);
        } else {
            return false;
        }
    }
    // }}}

    // {{{ singleton()
    /**
     * Attempts to return a reference to a concrete Turba_Driver instance
     * based on $driver. It will only create a new instance if no
     * Turba_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple preference sources (and, thus,
     * multiple Prefs instances) are required.
     *
     * This method must be invoked as: $var = &Turba_Driver::singleton()
     *
     * @param $driver   The type of concrete Turba_Driver subclass to return.
     *                  This is based on the storage driver ($driver). The
     *                  code is dynamically included.
     * @param $params   A hash containing additional parameters for the
     *                  subclass.
     *
     * @return          The concrete Turba_Driver reference, or false on an
     *                  error.
     */
    function &singleton($driver, $params)
    {
        static $instances;

        if (!isset($instances)) $instances = array();

        $sig = md5($driver . '][' . implode('][', $params));
        if (!isset($instances[$sig])) {
            $instances[$sig] = Turba_Driver::factory($driver, $params);
        }

        return $instances[$sig];
    }
    // }}}

    // {{{ getType()
    /**
     * Returns the name of the current driver type.
     *
     * @return          String containing the name of the current driver type.
     */
    function getType()
    {
        return $this->type;
    }
    // }}}

    // {{{ getParams()
    /**
     * Returns the current driver's additional parameters.
     *
     * @return          Hash containing the driver's additional parameters.
     */
    function getParams()
    {
        return $this->params;
    }
    // }}}

    // {{{ __lastNameCmp()
    /**
     * Compares two strings containing last names.
     *
     * @param $a        String containing the first last name.
     * @param $b        String containing the second last name.
     *
     * @return          Integer comparison of the two names from strcmp().
     */
    function lastNameCmp($a, $b)
    {
        $acmp = (isset($a['lastname'])) ? strtolower($a['lastname']) : '';
        $bcmp = (isset($b['lastname'])) ? strtolower($b['lastname']) : '';

        return strcmp($acmp, $bcmp);
    }
    // }}}

    // {{{ __emailCmp()
    /**
     * Compares two strings containing email addresses.
     *
     * @param $a        String containing the first email address.
     * @param $b        String containing the second email address.
     *
     * @return          Integer comparison of the two addresses from strcmp().
     */
    function emailCmp($a, $b)
    {
        $acmp = (isset($a['email'])) ? strtolower($a['email']) : '';
        $bcmp = (isset($b['email'])) ? strtolower($b['email']) : '';

        return strcmp($acmp, $bcmp);
    }
    // }}}

    // {{{
    /**
     * Compares two strings containing name aliases.
     *
     * @param $a        String containing the first name alias.
     * @param $b        String containing the second name alias.
     *
     * @return          Integer comparison of the two aliases from strcmp().
     */
    function aliasCmp($a, $b)
    {
        $acmp = (isset($a['alias'])) ? strtolower($a['alias']) : '';
        $bcmp = (isset($b['alias'])) ? strtolower($b['alias']) : '';

        return strcmp($acmp, $bcmp);
    }
    // }}}

    // {{{ guess_lastname()
    /**
     * Returns a best guess at the lastname in a string.
     *
     * @param $name     String contain the full name.
     *
     * @return          String containing the last name.
     */
    function guess_lastname($name)
    {
        if (!empty($name)) {
            /* Assume that last names are always before any commas. */
            if (is_int(strpos($name, ','))) {
                $name = substr($name, 0, strpos($name, ','));
            }

            $name = preg_replace('|\s+|', ' ', $name);
            $namelist = explode(' ', $name);
            $name = $namelist[($nameindex = (count($namelist) - 1))];

            while (strlen($name) < 5 &&
                   strspn($name[(strlen($name) - 1)], '.:-') &&
                   isset($namelist[($nameindex - 1)])) {
                $nameindex--;
                $name = $namelist[$nameindex];
            }
        }
        return $name;
    }
    // }}}

    // {{{ filter()
    /**
     * Filters the search results based on the specified sort routine.
     *
     * @param $addresses    Hash of addresses.
     * @param $sort         The sort method.
     * @param $low          The low end of the sort range.
     * @param $high         The high end of the sort range.
     *
     * @param               Hash containing the sorted range of addresses.
     */
    function filter($addresses, $sort = LASTNAME, $low = 'a', $high = 'z')
    {
        $low = strtolower($low);
        $high = strtolower($high);
        $range = array();

        foreach ($addresses as $address) {
            $address['lastname'] = $this->guess_lastname($address['fullname']);
            $chr = strtolower($address['lastname']);
            if (!$chr || (substr($chr, 0, strlen($low)) >= $low && substr($chr, 0, strlen($high)) <= $high))
                $range[] = $address;
        }

        if ($sort == LASTNAME) {
            usort($range, array($this, 'lastNameCmp'));
        } else if ($sort == EMAIL) {
            usort($range, array($this, 'emailCmp'));
        } else if ($sort == ALIAS) {
            usort($range, array($this, 'aliasCmp'));
        }

        return $range;
    }
    // }}}

    // {{{ error()
    /**
     * Returns a formatted error message.
     *
     * @return          The formatted error message.
     */
    function error()
    {
        $err = '';

        if ($this->errno) {
            $err .= '(' . $this->errno . ')';
        }
        if ($this->errstr) {
            if (!empty($err)) $err .= ' ';
            $err .= $this->errstr;
        }

        return $err;
    }
    // }}}

    /*
     * Abstract stubs to be implemented by subclasses.
     */
    function search($criteria)
    {
        // abstract
    }

    function delete($entry)
    {
        // abstract
    }

    function add($entry)
    {
        // abstract
    }

    function modify($entry)
    {
        // abstract
    }

}

?>
