<?php
// $Horde: turba/lib/Source.php,v 1.10 2000/09/15 15:37:36 chuck Exp $

require_once './lib/Turba.php';
require_once './lib/Driver.php';

/**
 * The Turba_Source:: class provides a set of methods for dealing with
 * specific contact sources.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.10 $
 * @since   Turba 0.0.1
 */
class Turba_Source {

    // {{{ properties

    /** String containing the internal name of this source. */
    var $name = '';

    /** String containing the symbolic title of this source. */
    var $title;

    /** Instance of the underlying Turba_Driver class. */
    var $driver;

    /** Hash describing the mapping between Turba attributes and
        driver-specific fields. */
    var $map = array();

    /** Boolean indicating whether this source is publicly searchable. */
    var $public = false;

    /** Boolean indicating whether this source is read-only (not editable). */
    var $readonly = true;

    // }}}

    // {{{ Turba_Source()
    /**
     * Constructs a new Turba_Source object.
     *
     * @param $name         String containing the internal name of this source.
     * @param $title        String containing the symbolic title of this source.
     * @param $driver       String indicating the underlying Turba_Driver
     *                      subclass.
     * @param $params       Additional parameters to pass to the driver
     *                      instance.
     * @param $map          Mapping between Turba attributes and
     *                      driver-specific fields.
     * @param $public       (optional) Boolean indicating whether this is a
     *                      public source.
     * @param $readonly     (optional) Boolean indicating whether this is a
     *                      read-only (non editable) source.
     */
    function Turba_Source($name, $title, $driver, $params, $map,
        $public = false, $readonly = true)
    {
        $this->name = $name;
        $this->title = $title;

        /*
         * Obtain a handle to a driver of the requested type.  If an
         * instance of that driver doesn't already exist, a new one will be
         * created and returned.
         */
        $this->driver = &Turba_driver::singleton($driver, $params);

        /* Store and translate the map at the Source level. */
        $this->map = $map;

        /* Set flags. */
        $this->public = $public;
        $this->readonly = $readonly;
    }
    // }}}

    // {{{ error()
    /**
     * Returns a formatted error message from the driver level.
     *
     * @return              String containing the formatted error message.
     */
    function error()
    {
        return $this->driver->error();
    }
    // }}}

    // {{{ toDriverKeys()
    /**
     * Translates the keys of the first hash from the generalized Turba
     * attributes to the driver-specific fields.  The translation is based
     * on the contents of $this->map.
     *
     * @param $hash         Hash using Turba keys.
     *
     * @return              Translated version of $hash.
     */
    function toDriverKeys($hash)
    {
        $fields = array();
        while (list($turba_key, $val) = each($hash)) {
            if (isset($this->map[$turba_key])) {
                $driver_key = $this->map[$turba_key];
                $fields[$driver_key] = $val;
            }
        }
        return $fields;
    }
    // }}}

    // {{{ toTurbaKeys()
    /**
     * Translates the keys of the first hash from the driver-specific
     * fields to the generalized Turba attributes.  The translation is based
     * on the contents of $this->map.
     *
     * @param $hash         Hash using driver-specific keys.
     *
     * @return              Translated version of $hash.
     */
    function toTurbaKeys($hash)
    {
        $attributes = array();
        for ($i = 0; $i < count($hash); $i++) {
            $entry = $hash[$i];
            $new_entry = array();

            reset($this->map);
            while (list($turba_key, $val) = each($this->map)) {
                $new_entry[$turba_key] = $entry[$val];
            }

            $attributes[] = $new_entry;
        }
        return $attributes;
    }
    // }}}

    // {{{ pickle()
    /**
     * Returns a hash of this object's current state for serialization.
     *
     * @return              Hash of this object's current state.
     */
    function pickle()
    {
        $state = array();
        $state['name'] = $this->name;
        $state['title'] = $this->title;
        $state['driver'] = $this->driver->getType();
        $state['params'] = $this->driver->getParams();
        $state['map'] = $this->map;
        $state['public'] = $this->public;
        $state['readonly'] = $this->readonly;

        return $state;
    }
    // }}}

    // {{{ unpickle()
    /**
     * Recreates a pickled instance of a Source object.
     * 
     * @param $state        The pickled state of the object.
     *
     * @return              The reinstated object.
     */
    function unpickle($state)
    {
        if (!is_array($state)) return false;

        return new Turba_Source($state['name'],
                                $state['title'],
                                $state['driver'],
                                $state['params'],
                                $state['map'],
                                $state['public'],
                                $state['readonly']);
    }
    // }}}

    // {{{ search()
    /**
     * Searches the source based on the provided criteria.
     *
     * TODO: Allow $criteria to contain the comparison operator (<, =, >,
     *       'like') and modify the drivers accordingly.
     *
     * @param $criteria     Hash containing the search criteria.
     * @param $sort         The requested sort order which is passed to
     *                      Turba_Driver::filter().
     *
     * @return              The filtered list of search results.
     */
    function search($criteria, $sort = LASTNAME)
    {
        global $turba;

        /*
         * If this is not a public source, enforce the requirement that the
         * source's key must be equal to the current user.
         */
        if (!$this->public) {
            $criteria['__key'] = $turba['user'];
        }

        /* Translate the Turba attributes to the driver-specific fields. */
        $fields = $this->toDriverKeys($criteria);

        /* Retrieve the search results from the driver. */
        $results = $this->driver->search($fields, array_values($this->map));
        
        /*
         * Translate the driver-specific fields in the result back to the 
         * more generalized common Turba attributes using the map.
         */
        $remapped_results = $this->toTurbaKeys($results);

        /* Return the filtered (sorted) results. */
        return $this->driver->filter($remapped_results, $sort);
    }
    // }}}

    // {{{ add()
    /**
     * Adds a new entry to the contact source.
     *
     * @param $entry        The new entry to add.
     */
    function add($entry)
    {
        if ($this->readonly) return false;

        // TODO: implementation
    }
    // }}}

    // {{{ delete()
    /**
     * Deletes the requested entry from the contact source.
     *
     * @param $entry        The entry to delete.
     */
    function delete($entry)
    {
        if ($this->readonly) return false;

        // TODO: implementation
    }
    // }}}

    // {{{ modify()
    /**
     * Modifies an existing entry in the contact source.
     *
     * @param $entry        The entry to modify.
     */
    function modify($entry)
    {
        if ($this->readonly) return false;

        // TODO: implementation
    }
}

?>
